import 'dart:core';
import 'dart:collection';
import "dart:typed_data";
import "../serialized/SerializedType.dart";
import "../fields/Field.dart";
import "../format/Format.dart";
import "../fields/Type.dart";
import "../serialized/BytesSink.dart";
import "../serialized/BinarySerializer.dart";
import "../serialized/TypeTranslator.dart";

import 'package:chainwallet/src/core/coretypes/uint/UInt16.dart';
import 'package:chainwallet/src/core/coretypes/uint/UInt32.dart';

import 'package:chainwallet/src/core/fields/UInt32Field.dart';
import 'package:chainwallet/src/core/serialized/BinaryParser.dart';
import 'package:chainwallet/src/core/coretypes/STObjectFormatter.dart';

import 'package:chainwallet/src/core/serialized/enums/TransactionType.dart';

import 'package:chainwallet/src/core/coretypes/AccountID.dart';
import 'package:chainwallet/src/core/coretypes/Amount.dart';
import 'package:chainwallet/src/core/coretypes/Blob.dart';

abstract class FieldFilter {
  bool evaluate(Field a);
}

class SerializedFilter implements FieldFilter {
  @override
  bool evaluate(Field a) {
    // TODO: implement evaluate
    return a.isSerialized();
  }
}

//
class SigningFilter implements FieldFilter {
  // @override
  bool evaluate(Field a) {
    // TODO: implement evaluate
    return a.isSerialized();
  }
}

class STObject implements SerializedType {
  //SplayTreeMap<Field,SerializedType>(Field.compare) fields;
  var fields = new SplayTreeMap<Field, SerializedType>(Field.compare);
  Format format;

  STObject();

  void checkFormat() => null;

  void setFormat(Format format) {
    this.format = format;
  }

  static TransactionType transactionType(STObject obj) {
    return obj.get(Field.TransactionType) as TransactionType;
  }

//  void computeFormat() {
//    UInt16 tt = get(UInt16.TransactionType);
//    if (tt != null) {
//      setFormat(TxFormat.fromNumber(tt));
//    }
//    UInt16 let = get(UInt16.LedgerEntryType);
//    if (let != null) {
//      setFormat(LEFormat.fromNumber(let));
//    }
//  }

  Map getFields() {
    return fields;
  }

  SerializedType get(Field field) {
    return fields[field];
  }

  UInt32 getUInt32(UInt32Field f) {
    return get(f.getField()) as UInt32;
  }

  SerializedType remove(Field f) {
    return fields.remove(f);
  }

  bool has(Field f) {
    return fields.containsKey(f);
  }

  @override
  Map toJSON() {
    return fields;
  }

  @override
  Uint8List toBytes() {
    return translate.toBytes(this);
  }

  @override
  String toHex() {
    return translate.toHex(this);
  }

  void toBytesSinkFilter(BytesSink to, FieldFilter p) {
    BinarySerializer serializer = new BinarySerializer(to);

    for (Field field in fields.keys) {
      // var tmp = field.name();

      if (p.evaluate(field)) {
        //   print('field   : $tmp');

//      System.out.println("Field:" + field.name());
        SerializedType value = fields[field];
        //  var valueTmp = value.toBytes();
        serializer.add(field, value);
      }
    }
  }

  @override
  void toBytesSink(BytesSink to) {
    toBytesSinkFilter(to, new SerializedFilter());
  }

  @override
  Type type() {
    return Type.STObject;
  }

  int size() {
    return fields.length;
  }

  void put(Field f, SerializedType value) {
    fields[f] = value;
  }

  static STObject formatted(STObject source) {
    return STObjectFormatter.doFormatted(source);
  }

  void putTranslated(Field f, Object value) {
    TypeTranslator typeTranslator = Translators.forField(f);

    SerializedType st = typeTranslator.fromValue(value);

//    var tmp = f.name();
//    print('putTranslated: $tmp');
//
//    if(st == null){
//
//      var test = 99;
//    }

    fields[f] = st;

//    try{
//
//
//    }catch(){
//
//    }

//    TypeTranslator typeTranslator = Translators.forField(f);
//    SerializedType st = null;
//    try {
//      st = typeTranslator.fromValue(value);
//    } catch (Exception e) {
//    throw new RuntimeException("Couldn't put `" +value+ "` into field `" + f + "`\n" + e.toString());
//    }
//    fields.put(f, st);
  }

  static STObjectTranslator translate = new STObjectTranslator();
}

class STObjectTranslator extends TypeTranslator<STObject> {
//  @override
  STObject fromParser(BinaryParser parser, int hint) {
    STObject so = new STObject();
    TypeTranslator<SerializedType> tr;
    SerializedType st;
    Field field;
    int sizeHint;

    // hint, is how many bytes to parse
    if (hint != null) {
      // end hint
      hint = parser.pos() + hint;
    }

    while (!parser.end(hint)) {
      field = parser.readField();
      if (field == Field.ObjectEndMarker) {
        break;
      }
      tr = Translators.forField(field);
      sizeHint = field.isVLEncoded() ? parser.readVLLength() : null;
      st = tr.fromParser(parser, sizeHint);
      if (st == null) {
        throw new Exception("Parsed " + field.toString() + " as null");
      }
      so.put(field, st);
    }

    return STObject.formatted(so);
  }

//
//  @override
//  Map toJSON(STObject obj) {
//    return toJSONObject(obj);
//  }
//
//  @override
//  JSONObject toJSONObject(STObject obj) {
//    JSONObject json = new JSONObject();
//
//    for (Field f : obj) {
//      SerializedType obj1 = obj.get(f);
//      Object object = obj1.toJSON();
//      json.put(f.name(), object);
//    }
//
//    return json;
//  }
//
//  @override
//  STObject fromJSONObject(JSONObject jsonObject) {
//    STObject so = new STObject();
//
//    Iterator keys = jsonObject.keys();
//    while (keys.hasNext()) {
//      String key = (String) keys.next();
//      Object value   = jsonObject.get(key);
//      Field fieldKey = Field.fromString(key);
//      if (fieldKey == null) {
//        continue;
//      }
//      so.putTranslated(fieldKey, value);
//    }
//    return STObject.formatted(so);
//  }
}

class Translators {
  static TypeTranslator forType(Type type) {
    if (identical(type, Type.STObject))
      return STObject.translate;
    else if (identical(type, Type.Amount))
      return Amount.translate;
    else if (identical(type, Type.UInt16))
      return UInt16.translate;
    else if (identical(type, Type.UInt32))
      return UInt32.translate;
    else if (identical(type, Type.AccountID))
      return AccountID.translate;
    else if (identical(type, Type.Blob))
      return Blob.translate;
    else
      throw new FormatException("Unknown type");

//    switch (type) {
//
//      case STObject:      return translate;
//      case Amount:        return Amount.translate;
//      case UInt16:        return UInt16.translate;
//      case UInt32:        return UInt32.translate;
//      case UInt64:        return UInt64.translate;
//      case Hash128:       return Hash128.translate;
//      case Hash256:       return Hash256.translate;
//      case Blob:          return Blob.translate;
//      case AccountID:     return AccountID.translate;
//      case STArray:       return STArray.translate;
//      case UInt8:         return UInt8.translate;
//      case Hash160:       return Hash160.translate;
//      case PathSet:       return PathSet.translate;
//      case Vector256:     return Vector256.translate;
//
//      default:            throw new FormatException("Unknown type");
//    }
  }

//
  static TypeTranslator<SerializedType> forField(Field field) {
    if (field.tag == null) {
      //TODO
      if (identical(field, Field.LedgerEntryType))
        field.tag = null;
      else if (identical(field, Field.TransactionType)) {
        field.tag = TransactionType.translate;
      } else {
        field.tag = forType(field.getType());
      }

//      switch (field) {
//        case LedgerEntryType:
//          field.tag = LedgerEntryType.translate;
//          break;
//        case TransactionType:
//          field.tag = TransactionType.translate;
//          break;
//        case TransactionResult:
//          field.tag = EngineResult.translate;
//          break;
//        default:
//          field.tag = forType(field.getType());
//          break;
//      }
    }
    return getCastedTag(field);
  }

//
// //@SuppressWarnings("unchecked")
  static TypeTranslator<SerializedType> getCastedTag(Field field) {
    return field.tag as TypeTranslator<SerializedType>;
  }
}
