import "dart:typed_data";

import 'package:chainwallet/src/core/serialized/SerializedType.dart';
import 'package:chainwallet/src/core/serialized/TypeTranslator.dart';

import 'package:chainwallet/src/utils/Util.dart';
import 'package:collection/collection.dart';

import 'package:chainwallet/src/core/serialized/BinaryParser.dart';

import 'package:chainwallet/src/core/serialized/BytesSink.dart';

import 'package:chainwallet/src/core/coretypes/hash/Hash256.dart';
import 'package:pointycastle/digests/sha512.dart';

import 'package:chainwallet/src/core/coretypes/hash/prefixes/Prefix.dart';

class HalfSha512 implements BytesSink {
  var sha512 = new SHA512Digest();

  HalfSha512() {}

  void addByte(int aByte) {
    sha512.updateByte(aByte);
  }

  void addBytes(Uint8List bytes) {
    sha512.update(bytes, 0, bytes.length);
  }

  Hash256 finish() {
    var out = new Uint8List(sha512.digestSize);
    ;
    sha512.doFinal(out, 0);
    var half512 = out.sublist(0, 32);

    return new Hash256(half512);
  }

  void update(Prefix prefix) {
    sha512.update(prefix.bytes(), 0, prefix.bytes().length);
  }

  void updateUint8List(Uint8List bytes) {
    sha512.update(bytes, 0, bytes.length);
  }

  static HalfSha512 prefixed256(Prefix bytes) {
    HalfSha512 halfSha512 = new HalfSha512();
    halfSha512.update(bytes);
    return halfSha512;
  }
}
