import "dart:typed_data";

import 'package:chainwallet/src/core/coretypes/uint/UInt.dart';
import "package:chainwallet/src/core/serialized/BytesSink.dart";
import 'package:chainwallet/src/core/serialized/TypeTranslator.dart';
import 'package:chainwallet/src/core/coretypes/hash/Hash.dart';
import 'package:chainwallet/src/core/fields/Type.dart';
import 'package:chainwallet/src/core/coretypes/hash/prefixes/Prefix.dart';

import 'package:chainwallet/src/core/coretypes/hash/HalfSha512.dart';
import 'package:chainwallet/src/utils/Util.dart';

class Hash256Translator extends HashTranslator<Hash256> {
  Hash256 newInstance(Uint8List i) => new Hash256(i);

  int byteWidth() => 32;
}

class Hash256 extends Hash<Hash256> {
  static Hash256Translator translate = new Hash256Translator();

  Hash256(Uint8List bytes) : super(bytes, 32) {}

  static Hash256 prefixedHalfSha512(Prefix prefix, Uint8List blob) {
    HalfSha512 messageDigest = HalfSha512.prefixed256(prefix);
    messageDigest.updateUint8List(blob);
    return messageDigest.finish();
  }

//  Map toJSON() =>  null;
  Uint8List toBytes() => translate.toBytes(this);
  String toHex() => translate.toHex(this);
  void toBytesSink(BytesSink to) => translate.toBytesSink(this, to);
  Map toJSON() => null;

//
//
  Type type() => Type.Hash256;

  String toString() {
    return Util.formatBytesAsHexString(toBytes());
  }
//
//  int getByteWidth() => 2;

  //static Hash256Field InvoiceID = hash256Field(Field.InvoiceID);

}
