import "dart:typed_data";

import 'package:chainwallet/src/core/serialized/SerializedType.dart';
import 'package:chainwallet/src/core/serialized/TypeTranslator.dart';

import 'package:chainwallet/src/utils/Util.dart';

import 'package:chainwallet/src/core/serialized/BinaryParser.dart';
import 'package:chainwallet/src/core/serialized/BytesSink.dart';
//  foo
// abstract public class UInt<Subclass extends UInt> extends Number implements SerializedType, Comparable<UInt> {

abstract class UInt<T> implements SerializedType {
  BigInt value;

  static BigInt Max8 = BigInt.parse("256"),
      Max16 = BigInt.parse("65536"),
      Max32 = BigInt.parse("4294967296"),
      Max64 = BigInt.parse("18446744073709551616");

  int getByteWidth();

  T instanceFrom(BigInt n);

  T shiftLeft(int n) {
    return instanceFrom(value << n);
  }

  void setValue(BigInt value) {
    this.value = value;
  }

  int longValue() {
    return value as int;
  }

  Uint8List toByteArray() {
    int length = getByteWidth();

    {
      Uint8List bytes = Util.createUint8ListFromBigInt(value);
      if (bytes[0] == 0) {
        if (bytes.length - 1 > length) {
          throw new FormatException('standard length exceeded for value');
          // throw new IllegalArgumentException("standard length exceeded for value");
        }

        Uint8List tmp = new Uint8List(length);
        //foo &?
        Util.arraycopy(
            bytes, 1, tmp, tmp.length - (bytes.length - 1), bytes.length - 1);
        //  System.arraycopy(bytes, 1, tmp, tmp.length - (bytes.length - 1), bytes.length - 1);
        return tmp;
      } else {
        if (bytes.length == length) {
          return bytes;
        }

        if (bytes.length > length) {
          throw new FormatException('standard length exceeded for value');
          // throw new IllegalArgumentException("standard length exceeded for value");
        }

        Uint8List tmp = new Uint8List(length);
        Util.arraycopy(bytes, 0, tmp, tmp.length - bytes.length, bytes.length);
        // System.arraycopy(bytes, 0, tmp, tmp.length - bytes.length, bytes.length);

        return tmp;
      }
    }
  }

  T or(UInt val) {
    return instanceFrom(value | val.value);
  }
}

abstract class UINTTranslator<T extends UInt> extends TypeTranslator<T> {
  // interface
  T newInstance(BigInt i);
  int byteWidth();

  //  T newInstance(BigInt i) => null;

  Object toJSON(T obj) {
    if (obj.getByteWidth() <= 4) {
      return obj.longValue();
    } else {
      return UInttoString(obj);
    }
  }

  T fromParser(BinaryParser parser, int hint) {
    // TODO  newInstance(new BigInteger(1, parser.read(byteWidth())));
    BigInt big = Util.createBigIntFromUint8List(parser.read(byteWidth()));
    return newInstance(big);
  }

  String UInttoString(T obj) {
    // return B16.toString(obj.toByteArray());
    return Util.formatBytesAsHexString(obj.toByteArray());
  }

  T fromInteger(int integer) {
    return newInstance(BigInt.from(integer));
  }

  void toBytesSink(T obj, BytesSink to) {
    to.addBytes(obj.toByteArray());
  }
}

// private BigInteger value;

//import "../serialized/SerializedType.dart";
////
////class STObject implements SerializedType{
////  var fields = Map();
////
////  Map toJSON(){
////
////    return null;
////
////  }
////
////  Uint8List toBytes(){
////
////    return null;
////
////  }
////  String toHex(){
////
////    return null;
////
////  }
//////  void toBytesSink(BytesSink to){
//////
//////  }
////  Type type(){
////
////  }
////}
