import 'dart:core';
import "dart:typed_data";

import 'package:chainwallet/src/core/coretypes/uint/UInt.dart';
import "package:chainwallet/src/core/serialized/BytesSink.dart";
import 'package:chainwallet/src/core/serialized/TypeTranslator.dart';
import 'package:chainwallet/src/core/fields/Type.dart';

class UInt64 extends UInt<UInt64> {
  static UInt64 ZERO = new UInt64(BigInt.from(0));

  static TypeTranslator<UInt64> translate = new UINTTranslator64();

  UInt64(BigInt value) {
    super.setValue(value);
  }

  UInt64 instanceFrom(BigInt n) {
    return new UInt64(n);
  }

  Map toJSON() => null;
  Uint8List toBytes() => translate.toBytes(this);
  String toHex() => translate.toHex(this);

  void toBytesSink(BytesSink to) => translate.toBytes(this);

  Type type() => Type.UInt64;

  int getByteWidth() => 8;
}

class UINTTranslator64 extends UINTTranslator<UInt64> {
  UInt64 newInstance(BigInt i) {
    return new UInt64(i);
  }

  int byteWidth() {
    return 8;
  }
}
