import 'dart:typed_data';

import "./Type.dart";

class Field {
  static final Generic = new Field(0, Type.Unknown, "Generic");

  static final LedgerEntryType = new Field(1, Type.UInt16, "LedgerEntryType");

  static final TransactionType = new Field(2, Type.UInt16, "TransactionType");

  static final SigningPubKey = new Field(3, Type.Blob, "SigningPubKey");
  static final TxnSignature = new Field(4, Type.Blob, "TxnSignature");

  static final Sequence = new Field(4, Type.UInt32, "Sequence");

  static final Amount = new Field(1, Type.Amount, "Amount");
  static final LimitAmount = new Field(3, Type.Amount, "LimitAmount");

  static final Fee = new Field(8, Type.Amount, "Fee");
  static final SendMax = new Field(9, Type.Amount, "SendMax");

  static final Account = new Field(1, Type.AccountID, "Account");
  static final Destination = new Field(3, Type.AccountID, "Destination");

  static final ObjectEndMarker = new Field(1, Type.STObject, "ObjectEndMarker");

  static final ArrayEndMarker = new Field(1, Type.STArray, "ArrayEndMarker");

  static final Paths = new Field(1, Type.PathSet, "Paths");
  static final InvoiceID = new Field(17, Type.Hash256, "InvoiceID");

  static final DestinationTag = new Field(14, Type.UInt32, "DestinationTag");
  static final QualityIn = new Field(20, Type.UInt32, "QualityIn");

  static final QualityOut = new Field(21, Type.UInt32, "QualityOut");

  static final Flags = new Field(2, Type.UInt32, "Flags");

  //Paths(1, Type.PathSet),

  final int id;

  String _name;

//  Field.SendMax,         Requirement.OPTIONAL,
//  Field.Paths,           Requirement.DEFAULT,
//  Field.InvoiceID,       Requirement.OPTIONAL,
//  Field.DestinationTag,  Requirement.OPTIONAL]);

  // defaults
  bool signingField = true;
  bool bSerialized = true;
  bool bVlEncoded = false;

  final int code;
  final Type type;
  Uint8List bytes;
  Object tag = null;

  Field(int fid, Type tid, String name)
      : id = fid,
        _name = name,
        type = tid,
        code = (tid.id << 16) | fid {
    if (isSerialized()) {
      bytes = asBytes(this);
    } else {
      bytes = null;
    }
    bSerialized = _isSerialized(this);

    if (tid == Type.AccountID || tid == Type.Blob) {
      bVlEncoded = true;
    }
  }

  String name() => _name;

  static Uint8List asBytes(Field field) {
    int name = field.getId(), type = field.getType().getId();

    List<int> header = new List<int>();

    if (type < 16) {
      if (name < 16) // common type, common name
        header.add(type << 4 | name);
      else {
        // common type, uncommon name
        header.add(type << 4);
        header.add(name);
      }
    } else if (name < 16) {
      // uncommon type, common name
      header.add(name);
      header.add(type);
    } else {
      // uncommon type, uncommon name
      header.add(0);
      header.add(type);
      header.add(name);
    }

    Uint8List headerBytes = Uint8List.fromList(header);
    return headerBytes;
  }

  static bool _isSerialized(Field f) {
    // This should screen out `hash` and `index` and the like
    return ((f.type.id > 0) && (f.type.id < 256) && (f.id > 0) && (f.id < 256));
  }

  static Map<int, Field> byCode = new Map<int, Field>();
  static bool _bInit = false;
  static Field fromCode(int integer) {
    if (!_bInit) {
      byCode[Field.Generic.code] = Field.Generic;

      byCode[Field.LedgerEntryType.code] = Field.LedgerEntryType;

      byCode[Field.TransactionType.code] = Field.TransactionType;
      byCode[Field.Sequence.code] = Field.Sequence;
      byCode[Field.Amount.code] = Field.Amount;

      byCode[Field.Fee.code] = Field.Fee;
      byCode[Field.SendMax.code] = Field.SendMax;
      byCode[Field.Account.code] = Field.Account;

      byCode[Field.LimitAmount.code] = Field.LimitAmount;

      byCode[Field.Destination.code] = Field.Destination;
      byCode[Field.ObjectEndMarker.code] = Field.ObjectEndMarker;
      byCode[Field.ArrayEndMarker.code] = Field.ArrayEndMarker;
      byCode[Field.SigningPubKey.code] = Field.SigningPubKey;

      byCode[Field.TxnSignature.code] = Field.TxnSignature;

      byCode[Field.Paths.code] = Field.Paths;
      byCode[Field.InvoiceID.code] = Field.InvoiceID;
      byCode[Field.DestinationTag.code] = Field.DestinationTag;

      byCode[Field.QualityIn.code] = Field.QualityIn;

      byCode[Field.QualityOut.code] = Field.QualityOut;

      byCode[Field.Flags.code] = Field.Flags;

      Field.Account.bVlEncoded = true;
      //Field.Vector256.bVlEncoded = true;
      //Field.Blob.bVlEncoded = true;

      _bInit = true;
    }

    return byCode[integer];
  }

  bool isSerialized() {
    return bSerialized;
  }

  bool isSigningField() {
    return signingField;
  }

  bool isVLEncoded() {
    return bVlEncoded;
  }

  int getId() => id;
  Type getType() => type;

  static int compare(Field key1, Field key2) {
    return key1.code - key2.code;
  }

  Uint8List getBytes() {
    return bytes;
  }
}
