import 'dart:core';
import "dart:typed_data";
import "../../coretypes/STObject.dart";
import "../../serialized/enums/TransactionType.dart";
import "../../format/Format.dart";
import "../../format/TxFormat.dart";
import "../../fields/Field.dart";
import "../../serialized/ByteList.dart";
import "../../coretypes/hash/prefixes/HashPrefix.dart";

import 'package:chainwallet/src/core/types/signed/SignedTransaction.dart';
import 'package:chainwallet/src/core/coretypes/Blob.dart';
import 'package:chainwallet/src/core/coretypes/uint/UInt32.dart';
import 'package:chainwallet/src/core/coretypes/enums/TransactionFlag.dart';

class Transaction extends STObject {
  static bool CANONICAL_FLAG_DEPLOYED = true;
  static UInt32 CANONICAL_SIGNATURE =
      new UInt32(BigInt.from(TransactionFlag.FullyCanonicalSig));

  Transaction(TransactionType type) {
    // TxFormat.Payment;

    setFormat(TxFormat.getTxFormat(type));
    put(Field.TransactionType, type);
  }

  Uint8List signingData() {
    BytesList bl = new BytesList();
    bl.addBytes(HashPrefix.txSign.bytes());

    var tmp = bl.bytes();
    // print("before : $tmp");

    toBytesSinkFilter(bl, new SigningFilter());
    //  tmp = bl.bytes();
    // print("After : $tmp");

    return bl.bytes();
  }

  void parseFromJson(Map obj) {
    Map<Field, Requirement> fieldMap = this.format.requirements();

    fieldMap.forEach((key, value) {
      String fieldName = key.name();
      if (obj.containsKey(fieldName)) {
        this.putTranslated(key, obj[fieldName]);

//        var tmp = toBytes();
//        var hexStr =  Util.formatBytesAsHexString(tmp);
//
//        print("$fieldName   HexStr : $hexStr");

      }
    });
  }

  SignedTransaction sign(String secret) {
    SignedTransaction signed = SignedTransaction.fromTx(this);

    signed.sign(secret);
    return signed;
  }

  TransactionType transactionType() {
    return STObject.transactionType(this);
  }

  void signingPubKeyFromBlob(Blob val) {
    put(Field.SigningPubKey, val);
  }

  void setCanonicalSignatureFlag() {
    UInt32 flags = getUInt32(UInt32.Flags);
    if (flags == null) {
      flags = CANONICAL_SIGNATURE;
    } else {
      flags = flags.or(CANONICAL_SIGNATURE);
    }

    // TODO  ?
    //put(UInt32.Flags, flags);
    put(UInt32.Flags.getField(), flags);
  }

  void txnSignature(Blob val) {
    put(Field.TxnSignature, val);
  }
}
