import "dart:typed_data";

import "package:base58check/base58.dart";
import 'package:collection/collection.dart';
import 'Util.dart';

class B58IdentiferCodecs {
  static const num VER_ACCOUNT_ID = 0;
  static const num VER_FAMILY_SEED = 33;

  static const num VER_NONE = 1;
  static const num VER_NODE_PUBLIC = 28;
  static const num VER_NODE_PRIVATE = 32;
  static const num VER_ACCOUNT_PUBLIC = 35;
  static const num VER_ACCOUNT_PRIVATE = 34;
  static const num VER_FAMILY_GENERATOR = 41;

  static String encode(Uint8List d, int version) {
    Uint8List buffer = Util.addList(Uint8List.fromList([version]), d);
    //  print('${result}');
    var dRes = Util.doubleDigest(buffer);

    //  print('${dRes}');
    var checkSum = dRes.sublist(0, 4);
    //var output = new Uint8List(buffer.length + checkSum.length);

    var output = Util.addList(buffer, checkSum);

    Base58Codec codec = const Base58Codec(Util.rippleAlphabet);
    //List<int> bytes = hexToBytes("21D2AE304B2E375ABEA165D35D570FCB0288289335");

    var encodeStr = codec.encode(output);

    return encodeStr;
  }

  static Uint8List decodeAddress(String address) {
    return decodeChecked(address, VER_ACCOUNT_ID);
  }

  static Uint8List decodeChecked(String input, int version) {
    Uint8List buffer = decodeAndCheck(input);
    int actualVersion = buffer[0];
    if (actualVersion != version) {
      throw new FormatException("Bro, version is wrong yo");
    }
    return buffer.sublist(1, buffer.length - 4);
  }

  static Uint8List decodeAndCheck(String input) {
    Base58Codec codec = const Base58Codec(Util.rippleAlphabet);
    var buffer = codec.decode(input);
    if (buffer.length < 4) {
      throw new FormatException('Bro, version is wrong yo');
    }

    var toHash = buffer.sublist(0, buffer.length - 4);
    var hashed = Util.doubleDigest(toHash).sublist(0, 4);
    var checksum = buffer.sublist(buffer.length - 4, buffer.length);

    Function eq = const ListEquality().equals;
    //  print(eq([1,'two',3], [1,'two',3])); // => true

    if (!eq(hashed, checksum)) {
      throw new FormatException('Bro, version is wrong yo');
    }
    return buffer;
  }

  static Map decodeMulti(
      String input, int expectedLength, List<Uint8List> possibleVersions) {
    Uint8List buffer = decodeAndCheck(input);
    if (buffer == null) return null;

    int versionLength = buffer.length - 4 - expectedLength;
    Uint8List versionBytes = buffer.sublist(0, versionLength);

    Function eq = const ListEquality().equals;
    Uint8List foundVersion = null;

    for (var possible in possibleVersions) {
      if (eq(possible, versionBytes)) {
        foundVersion = possible;
        break;
      }
    }

    if (foundVersion == null) {
      throw new FormatException('Bro, version is wrong yo');
    }

    var bytes = buffer.sublist(versionLength, buffer.length - 4);
    Map retMap = {"version": foundVersion, "payload": bytes};
    return retMap;
  }

  static String encodeFamilySeed(Uint8List bytes) {
    return encode(bytes, VER_FAMILY_SEED);
  }

  static String encodeAddress(Uint8List bytes) {
    return encode(bytes, VER_ACCOUNT_ID);
  }
}
